@@ -2,12 +2,21 @@
 
 ostream &operator<<(ostream &out, DFA const &dfa)
 {
-    size_t nCols = dfa.d_ranges->nUsed() + 3;
+    if (dfa.d_nUsedRanges == 0)
+    {
+        out << "    No transitions\n"
+                "\n";
+        return out;
+    }
+
+    size_t nCols = dfa.d_nUsedRanges + 2;
 
     TableLines support;
     support << 0;
-    for (size_t idx = 0; idx != nCols - 3; ++idx)   // set separator widths
+
+    for (size_t idx = 0; idx !=  nCols - 3; ++idx)   // set separator widths
         support << 2;
+
     support << 3 << 3 << 2;                         // F column stands out
                                                     // more clearly
 
@@ -18,12 +27,12 @@ ostream &operator<<(ostream &out, DFA const &dfa)
 
 
     table << "   ";                                // char-ranges display
-    for (size_t idx = 0, end = dfa.d_ranges->size(); idx++ != end; )
+    for (size_t idx = 0, end = dfa.d_ranges->nRanges(); idx++ != end; )
     {
-        if (dfa.d_ranges->used(idx))
+        if (dfa.d_usedRanges[idx - 1])
             table << idx;
     }
-    table << 'F' << 'A';
+    table << 'F';
                                                 // display rows of the table
     for 
     (
@@ -31,8 +40,6 @@ ostream &operator<<(ostream &out, DFA const &dfa)
             idx != end; 
                 ++idx)
     {
-//        if (dfa.d_unique[idx] < next)
-//            continue;
         table << next << dfa.d_row[idx];
         ++next;
     }
