@@ -4,12 +4,13 @@
 #include <iosfwd>
 #include <vector>
 #include <set>
-#include <unordered_map>
 
-#include "../ranges/ranges.h"
+#include <bobcat/linearmap>
+
 #include "../rules/rules.h"
 #include "../states/states.h"
-#include "../tailcount/tailcount.h"
+
+#include "../flextypes/flextypes.h"
 
 namespace FBB
 {
@@ -18,16 +19,14 @@ namespace FBB
 
 class States;
 
-class DFARow
+class DFARow: private FlexTypes
 {
     typedef std::pair<size_t, size_t> Pair;
     typedef std::set<size_t> StateSet;
     typedef std::vector<StateSet> StateSetVector;   // a set of states per
                                                     // input symbol
 
-    std::vector<TailCount> d_tailCount;         // tail counts for LOP rules
-
-    std::unordered_map<size_t, size_t> d_map;   // Relate input symbols (key) 
+    FBB::LinearMap<size_t, size_t> d_map;   // Relate input symbols (key) 
                                                 // to the row to transit to 
                                                 // (value)
 
@@ -43,10 +42,12 @@ class DFARow
                                             // of the States used for this
                                             // DFArow.
     Rules *d_rules;
-    Ranges *d_ranges;
+
+    size_t d_nRanges;
+    std::vector<bool> *d_usedRanges;
 
     public:
-        typedef std::unordered_map<size_t, size_t>::value_type MapValue;
+        typedef FBB::LinearMap<size_t, size_t>::value_type MapValue;
 
         DFARow() = default;
 
@@ -59,7 +60,8 @@ class DFARow
                 // index in the DFA (and in stateSets) of this row
             size_t thisIdx,
                 // info about used character ranges
-            Ranges &ranges
+            size_t nRanges,
+            std::vector<bool> *usedR
         );
 
             // visit all states of this row and determine their transitions
@@ -68,32 +70,28 @@ class DFARow
         void tabulate(FBB::Table &table) const;
 
 
-        std::unordered_map<size_t, size_t> const &map() const;
+        FBB::LinearMap<size_t, size_t> const &map() const;
         size_t size() const;            // the number of character-ranges
 
         std::string const &action(size_t idx) const;  // only for FINAL rows
 
-        std::pair<size_t, size_t> const &final() const; // final row for which
-                                                        // rules? the 1st
-                                                        // index is a BOL rule
-                                                        // the 2nd not (or
-                                                        // std::numeric_limits<size_t>::max())
-        std::vector<TailCount> &tailCounts();
+            // Rules matched when this row represents a final state are set
+            // in `transitions'. It calls the reverse mapping
+            // Rules:ruleFromFinalState to find the rule for which this is a
+            // final state.
+        std::pair<size_t, size_t> const &final() const; // final state for 
+                                        // which rules? the 1st value is a 
+                                        // non-BOL rule, the 2nd a BOL-rule
+                                        // (or max<size_t>() if not a final
+                                        // non-BOL/BOL state) 
 
-        std::vector<TailCount> const &tailCounts() const;
-             
         bool operator==(DFARow const &rhs) const;
 
         void uniqueMap(std::vector<size_t> const &xlat);
 
-        static void keepViableTailCounts(DFARow &dfaRow);
-
     private:
-        void updateViable(size_t &destIdx, size_t ruleIdx); // in setfinal.cc
-
         void tabulateTransitions(FBB::Table &table) const;
         void tabulateFinals(FBB::Table &table) const;
-        void tabulateAccepts(FBB::Table &table) const;
 
             // determine the eClosure of a set of transitions for each of the
             // char-ranges of the input alphabet, including the special 
@@ -107,30 +105,19 @@ class DFARow
 
         static void translate(MapValue &transition, 
                                             std::vector<size_t> const &xlat);
-        void probeTailCount(size_t stateIdx);
         static bool stateOfRule(size_t state, 
                                 std::vector<size_t> const &haystack);
 
         static bool sameTransits(
-            std::unordered_map<size_t, size_t> const &lhs,
-            std::unordered_map<size_t, size_t> const &rhs);
+            FBB::LinearMap<size_t, size_t> const &lhs,
+            FBB::LinearMap<size_t, size_t> const &rhs);
 };
 
-inline std::unordered_map<size_t, size_t> const &DFARow::map() const
+inline FBB::LinearMap<size_t, size_t> const &DFARow::map() const
 {
     return d_map;
 }
 
-inline std::vector<TailCount> &DFARow::tailCounts()
-{
-    return d_tailCount;
-}
-
-inline std::vector<TailCount> const &DFARow::tailCounts() const
-{
-    return d_tailCount;
-}
-
 inline std::pair<size_t, size_t> const &DFARow::final() const
 {
     return d_finalRule;
@@ -138,15 +125,11 @@ inline std::pair<size_t, size_t> const &DFARow::final() const
 
 inline size_t DFARow::size() const
 {
-    return d_ranges->size();
+    return d_nRanges;
 }
 
 FBB::Table &operator<<(FBB::Table& out, DFARow const &row);
 
-//        static void mergeFinalSet(DFARow &dfaRow);
-//        static void mergeFinal(size_t rule, 
-//                               std::vector<TailCount> &tailCount);
-        
 #endif
 
 
