@@ -11,7 +11,7 @@
 void DFARow::transitions()
 {
         // visit all ranges of input characters
-    for (size_t nr = 0, end = d_ranges->size(); nr++ != end; )    
+    for (size_t nr = 0; nr++ != d_nRanges; )    
     {
         StateSet &thisSet = (*d_stateSets)[d_thisIdx];
 
@@ -27,7 +27,7 @@ void DFARow::transitions()
         if (nextSet.empty())
             continue;
 
-        d_ranges->setUsed(nr);
+        (*d_usedRanges)[nr - 1] = true;
 
         auto iter = find(d_stateSets->begin(), d_stateSets->end(), nextSet);
 
@@ -43,11 +43,3 @@ void DFARow::transitions()
         d_map[nr] = idx;                // add the transition
     }
 }
-
-
-
-
-
-
-
-
