@@ -5,7 +5,6 @@ Generator::Map Generator::s_insert =
     // $insert-key              function called
     {"DFAbases",                &Generator::DFAbases},
     {"DFAs",                    &Generator::dfas},
-    {"RFCs",                    &Generator::rfcs},
     {"actions",                 &Generator::actions},
     {"baseclass_h",             &Generator::baseClassH},
     {"baseIncludes",            &Generator::baseIncludes},
@@ -24,6 +23,7 @@ Generator::Map Generator::s_insert =
     {"interactiveInit",         &Generator::interactiveInit},
     {"interactiveLex",          &Generator::interactiveLex},
     {"lexFunctionDecl",         &Generator::lexFunctionDecl},
+    {"lopImplementation",       &Generator::lopImplementation},
     {"namespace-close",         &Generator::namespaceClose},    
     {"namespace-open",          &Generator::namespaceOpen},
     {"namespace-use",           &Generator::namespaceUse},
@@ -32,7 +32,6 @@ Generator::Map Generator::s_insert =
     {"scannerConstructors",     &Generator::scannerConstructors},
     {"startCondNames",          &Generator::startCondNames},
     {"sstream",                 &Generator::insertSstream},
-    {"tailCount",               &Generator::tailCount},
 };
 
 char const *Generator::s_baseFlag = "\\@";
