@@ -8,12 +8,9 @@ void Generator::dfas(ostream &out) const
     "int const " << className << "Base::s_dfa__[][" << dfaCols() << "] =\n" 
     "{\n";
 
-    auto iter = d_dfas.find("INITIAL");
-    if (iter != d_dfas.end())
-        dfa(*iter, out, d_rfc, d_startStates, d_dfaIndices);
-
+    // write the DFAs: INITIAL is first, because FBB::LinearMap is used.
     for(auto &dfaPair: d_dfas) 
-        dfa(dfaPair, out, d_rfc, d_startStates, d_dfaIndices);
+        dfa(dfaPair, out, d_startStates, d_dfaIndices);
  
     out << "};\n";
 }
