@@ -5,21 +5,24 @@
 #include <string>
 #include <fstream>
 #include <set>
-#include <unordered_map>
 
+#include <bobcat/linearmap>
+
+#include "../flextypes/flextypes.h"
 #include "../dfas/dfas.h"
-#include "../startconditions/startconditions.h"
 
-class Ranges;
-class Options;
-class States;
 class Block;
+class DFARow;
+class Options;
+class Ranges;
+class Rules;
+class States;
 
-class Generator
+class Generator: public FlexTypes
 {
     enum
     {
-        RFC_INDICES = 2, // begin/end indices in the run-time RFC array
+        RF_INDICES = 2, // begin/end indices in the run-time RF array
     };
 
     enum Flags
@@ -30,15 +33,8 @@ class Generator
         BOL             = 8,    // Rule is a LOP rule
     };
 
-    struct RuleFlagCount    // (RFC)
-    {
-        size_t d_rule;
-        size_t d_flag;
-        size_t d_count;
-    };
-
     typedef void (Generator::*Inserter)(std::ostream &) const;
-    typedef std::unordered_map<std::string, Inserter>   Map;
+    typedef FBB::LinearMap<std::string, Inserter>   Map;
     typedef Map::value_type                             MapValue;
     typedef Map::const_iterator                         MapConstIter;
 
@@ -52,15 +48,15 @@ class Generator
     std::string d_baseclassScope;
     std::string d_constructionFilename;
 
-    bool d_lineDirectives;
+    mutable bool d_lineDirectives;          // manipulated by 'actions'
     bool d_debug;
+    bool d_lopUsed;
 
     mutable std::vector<std::string> d_startStates;
     mutable std::string d_key;          // extracted at $insert statements
     mutable std::string d_line;
     mutable std::string d_field;
 
-    mutable std::vector<RuleFlagCount> d_rfc;   // determined at dfas()
     mutable std::vector<size_t> d_dfaIndices;   // determined at dfas()
 
     static Map s_insert;
@@ -79,6 +75,7 @@ class Generator
     public:
         Generator(Rules const &rules, Ranges const &ranges, DFAs const &dfas);
 
+        bool conflicts() const;
         void construction(States const &states);
         void baseclassHeader() const;
         void classHeader() const;
@@ -89,15 +86,21 @@ class Generator
         void declarations();
 
     private:
+        void lopf(std::ostream &out) const;
+        void lop1(std::ostream &out) const;
+        void lop2(std::ostream &out) const;
+        void lop3(std::ostream &out) const;
+        void lop4(std::ostream &out) const;
+        void getLOP(std::ostream &out) const;
+
         static std::string filename(std::string const &path);
 
         void DFAbases(std::ostream &out) const;
         static std::string outDFAbase(std::string const &startState,
                                       size_t offset);
 
-        void warnExisting(std::string const &fileName, 
+        bool errExisting(std::string const &fileName, 
                           std::string const &option,
-                          std::string const &expectedOptionValue,
                           std::string const &regex) const;
 
         bool grep(std::string const &fileName, std::string const &regex) 
@@ -127,6 +130,7 @@ class Generator
         void interactiveLex(std::ostream &out) const;
         void key(std::ostream &out) const;
         void lexFunctionDecl(std::ostream &out) const;
+        void lopImplementation(std::ostream &out) const;
         void namespaceClose(std::ostream &out) const;
         void namespaceOpen(std::ostream &out) const;
         void namespaceUse(std::ostream &out) const;
@@ -134,28 +138,18 @@ class Generator
         void ranges(std::ostream &out) const;
         void scannerConstructors(std::ostream &out) const;
         void startCondNames(std::ostream &out) const;
-        void tailCount(std::ostream &out) const;
 
         size_t dfaCols() const;
 
-        void rfcs(std::ostream &out) const;
-        static void outRFC(RuleFlagCount const &rfc, std::ostream &out,
-                                                     size_t &idx);
         static void dfa(DFAs::Pair const &dfaPair, std::ostream &out, 
-                        std::vector<RuleFlagCount> &tailCounts,
                         std::vector<std::string> &startStates,
                         std::vector<size_t> &dfaOffsets);
-        static void dfaRow(DFARow const &row, size_t &index, 
-                        std::ostream &out, std::vector<RuleFlagCount> &rfc);
+        static void dfaRow(DFARow const &row, size_t &idx, std::ostream &out);
         static void dfaTransitions(DFARow const &row, std::ostream &out);
-        static void dfaRFCs(DFARow const &row, std::ostream &out,
-                            std::vector<RuleFlagCount> &rfc);
-        static void storeRFC(TailCount const &tailcount,
-                             std::pair<size_t, size_t> &final,
-                             std::vector<RuleFlagCount> &rfc);
+        static void dfaRules(DFARow const &row, std::ostream &out);
 
         static void outStartState(std::string const &name, std::ostream &out);
-        void ruleAction(Block const &block, std::ostream &out, size_t &idx)
+        void ruleAction(Block const &block, std::ostream &out, size_t idx)
                                                                     const;
 };
 
