@@ -2,6 +2,6 @@
 
 string Generator::outDFAbase(string const &startState, size_t offset)
 {
-    string ret = "    s_dfa__ + " + X2a(offset).str() + ",";
+    string ret = "    s_dfa__ + " + to_string(offset) + ",";
     return ret;
 }
