@@ -6,17 +6,17 @@ void Ranges::finalizeState(State &state)
 {
     switch (size_t type = state.type())
     {
-        case State::CHARSET:
-            charsetToRanges(state.data(), d_ranges);
+        case CHARSET:
+            charsetToRanges(state.data(), d_alphabet);
         break;
 
-        case State::EOF__:
+        case EOF__:
             state.setType(d_eof);
         break;
 
         default:
-            if (type < State::UNDETERMINED__)
-                state.setType(d_ranges[type]);
+            if (type < UNDETERMINED__)
+                state.setType(d_alphabet[type]);
         break;
     }
 }
