@@ -2,18 +2,27 @@
 
 void Scanner::maybeSwitchStream()
 {
+    bool isComment = false;
+
     string text = String::trim(matched().substr(sizeof("//include")));
 
     if (text[0] == '"' && *text.rbegin() == '"')    // "s around the name
         text = text.substr(1, text.length() - 2);   // rm the "s
-    
-    else if (text.find_first_of(" \t") != string::npos) // blanks need "s
+    else                                            // blanks in the name, but
+        isComment = text.find_first_of(" \t")       // no surrounding "s
+                    != string::npos;
+
+    if (not isComment)
+        isComment = access(text.c_str(), R_OK) != 0;// the file must exist
+                                                    // or //incl. is comment
+    if (isComment)
     {
         wmsg << '`' << matched() << "' considered comment" << endl;
         return;
     }
 
-cout << "SWITCHING TO STREAM " << text << '\n';
     setLineTags(text);
     pushStream(text);
 }
+
+
