@@ -13,24 +13,18 @@ $insert namespace-open
     //           class header file
 $insert ranges
 
-    // s_dfa__ contains the rows of *all* DFAs ordered by start state.
-    // The enum class StartCondition__ is defined in the baseclass header
-    // INITIAL is always 0.
-    // Each entry defines the row to transit to if the column's
-    // character range was sensed. Row numbers are relative to the
-    // used DFA and d_dfaBase__ is set to the first row of the subset to use.
-    // The row's final two values are begin and end indices in
-    // s_rfc__[] (rule, flags and count), defining the state's rule details
+    // s_dfa__ contains the rows of *all* DFAs ordered by start state.  The
+    // enum class StartCondition__ is defined in the baseclass header
+    // StartCondition__::INITIAL is always 0.  Each entry defines the row to
+    // transit to if the column's character range was sensed. Row numbers are
+    // relative to the used DFA, and d_dfaBase__ is set to the first row of
+    // the subset to use.  The row's final two values are respectively the
+    // rule that may be matched at this state, and the rule's FINAL flag. If
+    // the final value equals FINAL (= 1) then, if there's no continuation,
+    // the rule is matched. If the BOL flag (8) is also set (so FINAL + BOL (=
+    // 9) is set) then the rule only matches when d_atBOL is also true.
 $insert DFAs
 
-    // The first value is the rule index
-    // The second value is the FLAG: see the scannerbase.h file
-    // 1: Final     4: Count        11: Final/BOL,Inc     
-    // 2: Inc.      5: Final,Count  13: Final/BOL,Count
-    // 3: Final,Inc 9: Final/BOL    
-    // The third value is the LOP count value (valid for Count flags)
-$insert RFCs
-
 $insert DFAbases
 
 size_t \@Base::s_istreamNr = 0;
@@ -40,12 +34,7 @@ $insert inputImplementation
 \@Base::\@Base(std::istream &in, std::ostream &out)
 :
     d_filename("-"),
-    d_startCondition(StartCondition__::INITIAL),
-    d_state(0),
     d_out(new std::ostream(out.rdbuf())),
-    d_sawEOF(false),
-    d_atBOL(true),
-$insert tailCount
 $insert interactiveInit
     d_dfaBase__(s_dfa__)
 {}
@@ -53,10 +42,7 @@ $insert interactiveInit
 void \@Base::switchStream__(std::istream &in, size_t lineNr)
 {
     d_input.close();
-    d_state = 0;
     d_input = Input(new std::istream(in.rdbuf()), lineNr);
-    d_sawEOF = false;
-    d_atBOL = true;
 }
 
 $ignoreInteractive BEGIN    this section is ignored by generator/filter.cc
@@ -64,14 +50,9 @@ $ignoreInteractive BEGIN    this section is ignored by generator/filter.cc
 \@Base::\@Base(std::string const &infilename, std::string const &outfilename)
 :
     d_filename(infilename),
-    d_startCondition(StartCondition__::INITIAL),
-    d_state(0),
     d_out(outfilename == "-"    ? new std::ostream(std::cout.rdbuf()) :
           outfilename == ""     ? new std::ostream(std::cerr.rdbuf()) :
                                   new std::ofstream(outfilename)),
-    d_sawEOF(false),
-    d_atBOL(true),
-$insert tailCount
     d_input(new std::ifstream(infilename)),
     d_dfaBase__(s_dfa__)
 {}
@@ -115,7 +96,6 @@ void \@Base::switchIstream(std::string const &infilename)
     d_input.close();
     d_filename = infilename;
     d_input = Input(new std::ifstream(infilename));
-    d_sawEOF = false;
     d_atBOL = true;
 }
 
@@ -156,11 +136,9 @@ void \@Base::p_pushStream(std::string const &name, std::istream *streamPtr)
     d_streamStack.push_back(StreamStruct{d_filename, d_input});
     d_filename = name;
     d_input = Input(streamPtr);
-    d_sawEOF = false;
     d_atBOL = true;
 }
 
-
 bool \@Base::popStream()
 {
     d_input.close();
@@ -173,11 +151,15 @@ bool \@Base::popStream()
     d_input =   top.pushedInput;
     d_filename = top.pushedName;
     d_streamStack.pop_back();
-    d_sawEOF = false;
 
     return true;
 }
 
+$insert lopImplementation
+
+
+  // See the manual's section `Run-time operations' section for an explanation
+  // of this member.
 \@Base::ActionType__ \@Base::actionType__(size_t range)
 {
     d_nextState = d_dfaBase__[d_state][range];
@@ -185,8 +167,8 @@ bool \@Base::popStream()
     if (d_nextState != -1)                  // transition is possible
         return ActionType__::CONTINUE;
 
-    if (atFinalState())                     // FINAL state reached
-        return ActionType__::MATCH;
+    if (knownFinalState())                  // FINAL state reached
+        return ActionType__::MATCH;         
 
     if (d_matched.size())
         return ActionType__::ECHO_FIRST;    // no match, echo the 1st char
@@ -206,59 +188,69 @@ void \@Base::accept(size_t nChars)          // old name: less
     }
 }
 
-  // The size of d_matched is determined:
-  //    The current state is a known final state (as determined by 
-  // inspectRFCs__(), just prior to calling matched__). 
-  //    The contents of d_matched are reduced to d_final's size or (if set)
-  // to the LOP-rule's tail size.
-void \@Base::determineMatchedSize(FinData const &final)
+void \@Base::setMatchedSize(size_t length)
 {
-    size_t length = final.matchLen;
-    if (final.tailCount != std::numeric_limits<size_t>::max())
-        length -= final.tailCount;
-
-    d_input.reRead(d_matched, length);      // reread the tail section
-    d_matched.resize(length);               // return what's left
+    d_input.reRead(d_matched, length);  // reread the tail section
+    d_matched.resize(length);           // return what's left
 }
 
   // At this point a rule has been matched.  The next character is not part of
   // the matched rule and is sent back to the input.  The final match length
   // is determined, the index of the matched rule is determined, and then
-  // d_atBOL is updated. Finally the rule index is returned.
+  // d_atBOL is updated. Finally the rule's index is returned.
+  // The numbers behind the finalPtr assignments are explained in the 
+  // manual's `Run-time operations' section.
 size_t \@Base::matched__(size_t ch)
 {
 $insert 4 debug "MATCH"
     d_input.reRead(ch);
 
-    if (!d_atBOL)
-        d_final.atBOL.rule = std::numeric_limits<size_t>::max();
+    FinalData *finalPtr;
+                            
+    if (not d_atBOL)                    // not at BOL
+        finalPtr = &d_final.std;        // then use the std rule (3, 4)
 
-    FinData &final = d_final.atBOL.rule == std::numeric_limits<size_t>::max() ? 
-                            d_final.notAtBOL
-                        :
-                            d_final.atBOL;
+                                        // at BOL
+    else if (not available(d_final.std.rule))   // only a BOL rule avail.
+            finalPtr = &d_final.bol;            // use the BOL rule (6)
 
-    determineMatchedSize(final);
+    else if (not available(d_final.bol.rule)) // only a std rule is avail.
+        finalPtr = &d_final.std;        // use the std rule (7)
+        
+    else if (                           // Both are available (8)
+        d_final.bol.length !=           // check lengths of matched texts
+        d_final.std.length              // unequal lengths, use the rule
+    )                                   // having the longer match length
+        finalPtr =              
+            d_final.bol.length > d_final.std.length ?
+                &d_final.bol
+            :
+                &d_final.std;
 
-    d_atBOL = *d_matched.rbegin() == '\n';
+    else                            // lengths are equal: use 1st rule
+        finalPtr = 
+            d_final.bol.rule < d_final.std.rule ?
+                &d_final.bol
+            :
+                &d_final.std;
+
+    setMatchedSize(finalPtr->length);
+
+    d_atBOL = d_matched.back() == '\n';
 
 $insert 4 debug "match buffer contains `" << d_matched << "'"
 
-    return final.rule;
+    return finalPtr->rule;
 }
 
 size_t \@Base::getRange__(int ch)       // using int to prevent casts
 {
-    if (ch != AT_EOF)
-        d_sawEOF = false;
-
 $insert caseCheck
-    return ch == AT_EOF ? static_cast<size_t>(s_rangeOfEOF__) : s_ranges__[ch];
+    return ch == AT_EOF ? as<size_t>(s_rangeOfEOF__) : s_ranges__[ch];
 }
 
   // At this point d_nextState contains the next state and continuation is
-  // possible. The just read char. is appended to d_match, and LOP counts
-  // are updated.
+  // possible. The just read char. is appended to d_match
 void \@Base::continue__(int ch)
 {
 $insert 4 debug "CONTINUE, NEXT STATE: " << d_nextState
@@ -271,7 +263,7 @@ $insert 4 debug "CONTINUE, NEXT STATE: " << d_nextState
 void \@Base::echoCh__(size_t ch)
 {
 $insert 4 debug "ECHO_CH" 
-    *d_out << static_cast<char>(ch);
+    *d_out << as<char>(ch);
     d_atBOL = ch == '\n';
 }
 
@@ -289,46 +281,36 @@ $insert 4 debug "ECHO_FIRST"
     echoCh__(d_matched[0]);
 }
 
-    // Inspect all s_rfc__ elements associated with the current state
-    //  If the s_rfc__ element has its COUNT flag set then set the 
-    // d_tailCount[rule] value to the element's tailCount value, if it has its 
-    // INCREMENT flag set then increment d_tailCount[rule]
-    //  If neither was set set the d_tailCount[rule] to numeric_limits<size_t>::max()
-    // 
-    // If the s_rfc__ element has its FINAL flag set then store the rule number
-    // in d_final.second. If it has its FINAL + BOL flags set then store the
-    // rule number in d_final.first
-void \@Base::inspectRFCs__()
+    // Update the rules associated with the current state, do this separately
+    // for BOL and std rules.
+    // If a rule was set, update the rule index and the current d_matched
+    // length. 
+void \@Base::updateFinals__()
 {
-    for 
-    (
-        size_t begin = d_dfaBase__[d_state][s_finacIdx__], 
-                 end = d_dfaBase__[d_state][s_finacIdx__ + 1];
-            begin != end;
-                ++begin
-    )
+    size_t len = d_matched.size();
+
+    int const *rf = d_dfaBase__[d_state] + s_finIdx__;
+
+    if (rf[0] != -1)        // update to the latest std rule
     {
-        size_t const *rfc = s_rfc__[begin];
-        size_t flag = rfc[FLAGS];
-        size_t rule = rfc[RULE];
+$insert 4 debug "latest std rule: " << rf[0] << ", len = " << len
+        d_final.std = FinalData { as<size_t>(rf[0]), len };
+    }
 
-        if (flag & INCREMENT)
-            ++d_tailCount[rule];
-        else 
-            d_tailCount[rule] = (flag & COUNT) ? rfc[ACCCOUNT] : std::numeric_limits<size_t>::max();
-
-        if (flag & FINAL)
-        {
-            FinData &final = (flag & BOL) ? d_final.atBOL : d_final.notAtBOL;
-            final = FinData { rule, d_matched.size(), d_tailCount[rule] };
-        }
+    if (rf[1] != -1)        // update to the latest bol rule
+    {
+$insert 4 debug "latest BOL rule: " << rf[0] << ", len = " << len
+        d_final.bol = FinalData { as<size_t>(rf[1]), len };
     }
 }
 
 void \@Base::reset__()
 {
-    d_final = Final { {std::numeric_limits<size_t>::max(), std::numeric_limits<size_t>::max(), std::numeric_limits<size_t>::max() }, 
-                      {std::numeric_limits<size_t>::max(), std::numeric_limits<size_t>::max(), std::numeric_limits<size_t>::max() } };
+    d_final = Final{ 
+                    FinalData{s_unavailable, 0}, 
+                    FinalData {s_unavailable, 0} 
+                };
+
     d_state = 0;
     d_return = true;
 
@@ -365,7 +347,7 @@ int \@::lex__()
         size_t ch = get__();                // fetch next char
         size_t range = getRange__(ch);      // determine the range
 
-        inspectRFCs__();                    // update d_tailCount values
+        updateFinals__();                    // update the state's Final info
 
         switch (actionType__(range))        // determine the action
         {
