@@ -1,5 +1,5 @@
-#ifndef \@_H_INCLUDED_
-#define \@_H_INCLUDED_
+#ifndef \@$_H_INCLUDED_
+#define \@$_H_INCLUDED_
 
 $insert baseclass_h
 
@@ -16,6 +16,10 @@ $insert 8 lexFunctionDecl
         void print();
         void preCode();     // re-implement this function for code that must 
                             // be exec'ed before the patternmatching starts
+
+        void postCode(PostEnum__ type);    
+                            // re-implement this function for code that must 
+                            // be exec'ed after the rules's actions.
 };
 
 $insert scannerConstructors
@@ -27,6 +31,11 @@ inline void \@::preCode()
     // optionally replace by your own code
 }
 
+inline void \@::postCode(PostEnum__ type) 
+{
+    // optionally replace by your own code
+}
+
 inline void \@::print() 
 {
     print__();
