@@ -1,28 +1,11 @@
 #include "state.ih"
 
-#include <ostream>
-#include <algorithm>
-#include <iterator>
-
-namespace
-{
-    char const *flagName[] = 
-    {
-        "NO_LOP ",
-        "ACCEPT ",
-        "PRE    ",
-        "POST   "
-    };
-}
-
 ostream &operator<<(ostream &out, State const &state)
 {
     size_t type = state.type();
 
     if (type == State::UNDETERMINED__)
-        return(out);
-
-    out << flagName[state.flag()];
+        return out;
 
     if (type < State::UNDETERMINED__)
         out << type;
@@ -33,7 +16,8 @@ ostream &operator<<(ostream &out, State const &state)
     else if (type == State::CHARSET)
         out << 'S';
     else
-        out << '*' << type << '*';
+        fmsg << "internal error: * unexpected State type " << type << 
+                "encountered* in State's operator<<" << endl;
 
     out  << " -> " << state.next1() << ", " << state.next2();
 
@@ -52,3 +36,5 @@ ostream &operator<<(ostream &out, State const &state)
 
 
 
+
+
